INPUT1 = """{ 
    "SendingSite": "SITE1", 
    "SentTimestamp": "1502815271", 
    "PayloadType": "Alert", 
    "PayloadFormat": "STIX", 
    "DataSensitivity": "noSensitivity", 
    "SharingRestrictions": "WHITE", 
    "ReconPolicy": "noTouch"
}"""